#!/bin/sh

PATH=$PATH:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin

BSP_INST_HOME_PATH=$(dirname $(realpath $0))
BSP_ROOTFS_PATH=$BSP_INST_HOME_PATH/rootfs
TARGET_ROOTFS_PATH=

#CONFIG_FILE=/boot/config.txt
CONFIG_FILE=$(lsblk | grep mmcblk0p1 |sed 's/^.*part //')/config.txt
BOOT_PATH=$(lsblk | grep mmcblk0p1 |sed 's/^.*part //')
BLACKLIST=/etc/modprobe.d/raspi-blacklist.conf

CM4_IND_HW_MODEL_V1P1="Rev1.1"
CM4_IND_HW_MODEL_V1P3="Rev1.3"

set_antenna() {
    local conf="$CONFIG_FILE"
    local ant2="dtparam=ant2"

    grep -q "#.*${ant2}" $conf
    if [ $? -eq 0 ];then
        sed -i "s/#.*${ant2}/${ant2}/" $conf
    else
        echo "${ant2}" >> $conf
    fi
}

config_set() {
    local key="$1"
    local value="$2"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh set $key $value $conf
}

config_clear() {
    local key="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh clear $key $conf
}

config_get() {
    local key="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh get $key $conf
}

config_overlay_set() {
    local dtbo="$1"
    local params="$2"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh overlay_set $dtbo $conf $params
}

config_overlay_clear() {
    local dtbo="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh overlay_clear $dtbo $conf
}

config_overlay_rm() {
    local dtbo="$1"
    local conf="$CONFIG_FILE"

    ${BSP_INST_HOME_PATH}/raspi-conf.sh overlay_remove $dtbo $conf
}

module_load() {
    local name="$1"

    sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*$name\)/#\1/"
    sed /etc/modules -i -e "s/^#[[:space:]]*\($name\)/\1/"

    if ! grep -q "^$name" /etc/modules; then
        printf "$name\n" >> /etc/modules
    fi
}

module_load_i2c() {
    sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*i2c[-_]bcm2708\)/#\1/"
    sed /etc/modules -i -e "s/^#[[:space:]]*\(i2c[-_]dev\)/\1/"
    if ! grep -q "^i2c[-_]dev" /etc/modules; then
        printf "i2c-dev\n" >> /etc/modules
    fi
}

module_load_spi() {
    sed $BLACKLIST -i -e "s/^\(blacklist[[:space:]]*spi[-_]bcm2708\)/#\1/"
}

do_modules() {
    if ! [ -e $BLACKLIST ]; then
        touch $BLACKLIST
    fi

    module_load_i2c
    module_load_spi
    module_load industrialio
    module_load mtdblock
}

do_config_txt() {
    local hw_model="$1"

    config_set dtparam=i2c_arm on
    config_set dtparam=spi on
    set_antenna

    config_overlay_set spi1-1cs cs0_pin=16

    config_overlay_set jedec-spi-nor flash-spi1-0

    config_overlay_set i2c-rtc pcf8563

    if [ "$hw_model" = "$CM4_IND_HW_MODEL_V1P3" ]; then
        # full scale of the channel to 4.096 Volts
        config_overlay_set ads1015 cha_gain=1,chb_gain=1,chc_gain=1,chd_gain=1
        # Alert/RDY to GPIO Input, Pull up
        config_set gpio=7 "ip,pu"
    else
        config_overlay_set stmpe811
    fi

    config_overlay_set uart3
    config_overlay_set uart4
    config_overlay_set uart5

    config_set otg_mode 1

    # Button(GPIO6), input, pull up
    config_set gpio=6 "ip,pu"
}

do_install() {
    cp $BSP_ROOTFS_PATH/boot/dt-blob.bin $TARGET_ROOTFS_PATH$BOOT_PATH
    cp $BSP_ROOTFS_PATH/boot/overlays/*.dtbo $TARGET_ROOTFS_PATH$BOOT_PATH/overlays/
}

do_add_udev_rules() {
    local udev_dir=/etc/udev/rules.d
    local udev_file=50-local.rules
    local os_version=$(cat /etc/os-release | grep VERSION= | sed -e "s/.*(//; s/).*//")
    [ ${os_version} ] || echo "Read OS version Failed!"
    case $os_version in
    bullseye | buster)
        cat << EOF > ${udev_dir}/${udev_file}
KERNEL=="ttyAMA1", SYMLINK+="rs485-2"
KERNEL=="ttyAMA2", SYMLINK+="rs485-1"
KERNEL=="ttyAMA3", SYMLINK+="rs232"
EOF
        ;;
    bookworm)
        cat << EOF > ${udev_dir}/${udev_file}
KERNEL=="ttyAMA3", SYMLINK+="rs485-2"
KERNEL=="ttyAMA4", SYMLINK+="rs485-1"
KERNEL=="ttyAMA5", SYMLINK+="rs232"
EOF
        ;;
    *)
        echo "Unknown system version"
        ;;
    esac
}

CM4_IND_HW_MODEL="$1"

echo "CM4-Industrial hardware model: [$CM4_IND_HW_MODEL] ..."

echo "do install files ..."
do_install

echo "do config.txt ..."
do_config_txt $CM4_IND_HW_MODEL

echo "do modules ..."
do_modules

echo "add udev rules"
do_add_udev_rules


echo "done."

exit 0
