#
#!/bin/sh
#
# Created by Yongjia Niu <ahnniu@gmail.com>
#

set_config_var() {
  lua - "$1" "$2" "$3" <<EOF > "$3.bak"
local key=assert(arg[1])
local value=assert(arg[2])
local fn=assert(arg[3])
local file=assert(io.open(fn))
local made_change=false
for line in file:lines() do
  if line:match("^#?%s*"..key.."=.*$") then
    line=key.."="..value
    made_change=true
  end
  print(line)
end
if not made_change then
  print(key.."="..value)
end
EOF
mv "$3.bak" "$3"
}

clear_config_var() {
  lua - "$1" "$2" <<EOF > "$2.bak"
local key=assert(arg[1])
local fn=assert(arg[2])
local file=assert(io.open(fn))
for line in file:lines() do
  if line:match("^%s*"..key.."=.*$") then
    line="#"..line
  end
  print(line)
end
EOF
mv "$2.bak" "$2"
}

get_config_var() {
  lua - "$1" "$2" <<EOF
local key=assert(arg[1])
local fn=assert(arg[2])
local file=assert(io.open(fn))
local found=false
for line in file:lines() do
  local val = line:match("^%s*"..key.."=(.*)$")
  if (val ~= nil) then
    print(val)
    found=true
    break
  end
end
if not found then
   print(0)
end
EOF
}

get_overlay() {
    local key="$1"
    local conf="$2"
  if grep -q ^dtoverlay=$key $conf ; then
    echo 0
  else
    echo 1
  fi
}

clear_overlay() {
    local key="$1"
    local conf="$2"

    sed -i -e "s/^dtoverlay=$key/#dtoverlay=$key/g" $conf
}

remove_overlay() {
    local key="$1"
    local conf="$2"

    sed  -i "/dtoverlay=$key/d" $conf
}

set_overlay() {
    local key="$1"
    local conf="$2"
    local params="$3"
    local found=

    found=$(get_overlay $key $conf)

    if [ $found -eq 0 ]; then
        remove_overlay $key $conf
    fi

    if [ -z "$params" ]; then
        echo "dtoverlay=$key" >> $conf
    else
        echo "dtoverlay=$key,$params" >> $conf
    fi
}


usage() {
    echo "NAME"
    echo "    $(basename $0) - Raspberry Pi config.txt get / set / clear"
    echo ""
    echo "COMMANDS"
    echo "    set [key] [value] [filename]"
    echo ""
    echo "    clear [key] [filename]"
    echo ""
    echo "    get [key] [filename]"
    echo ""
    echo "    overlay_set [key] [filename] [params]"
    echo ""
    echo "    overlay_clear [key] [filename]"
    echo ""
}

if [ $# -eq 0 ] ; then
  usage
  exit 1
fi

TO_DO="$1"
ITEM_KEY="$2"
ITEM_VALUE="$3"
CONFIG_FILE="$4"

case ${TO_DO} in
"set" )
    set_config_var $2 $3 $4
    ;;
"clear" )
    clear_config_var $2 $3
    ;;
"get" )
    get_config_var $2 $3
    ;;
"overlay_set" )
    set_overlay $2 $3 $4
    ;;
"overlay_clear" )
    clear_overlay $2 $3
    ;;
"overlay_remove" )
    remove_overlay $2 $3
    ;;
*)
    usage
    exit 1
    ;;
esac
